import unittest

from src.json_criteria.internal import meets_crit

class TestMeetsCrit(unittest.TestCase):
    def test_basic(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = { 'key': 'age', 'op': 'equal_to', 'value': 30 }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_basic2(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = { 'AND': [
            {'key': 'age', 'op': 'equal_to', 'value': 30 },
            {'key': 'name', 'op': 'equal_to', 'value': 'Joe' }
            ]
        }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_basic3(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = { 'AND': [
            {'key': 'age', 'op': 'equal_to', 'value': 300 },
            {'key': 'name', 'op': 'equal_to', 'value': 'Joe' }
            ]
        }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_single_cond_true(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'equal_to', 'value': 30 }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_single_cond_false(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'equal_to', 'value': 29 }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_AND_true(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = { 'AND': [
            {'key': 'id', 'op': 'equal_to', 'value': 1 },
            {'key': 'email', 'op': 'equal_to', 'value': 'test@email.com' },
            {'key': 'is_active', 'op': 'equal_to', 'value': True }
            ]
        }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_AND_false(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = { 'AND': [
            {'key': 'id', 'op': 'equal_to', 'value': 1 },
            {'key': 'email', 'op': 'equal_to', 'value': 'test@email.com' },
            {'key': 'is_active', 'op': 'equal_to', 'value': False }
            ]
        }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_OR_true(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = { 'OR': [
            {'key': 'id', 'op': 'greater_than', 'value': 1 },
            {'key': 'email', 'op': 'equal_to', 'value': 'test@email.com' },
            ]
        }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_OR_false(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = { 'OR': [
            {'key': 'id', 'op': 'greater_than', 'value': 1 },
            {'key': 'is_active', 'op': 'equal_to', 'value': False },
            ]
        }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_NOT_true(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = { 'NOT': {'key': 'id', 'op': 'greater_than', 'value': 1 } }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_NOT_false(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = { 'NOT': {'key': 'id', 'op': 'greater_than', 'value': 0 } }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_combination_1_true(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = {
            'NOT': {
                'AND': [
                    { 'key': 'id', 'op': 'greater_than', 'value': 1 },
                    { 'key': 'id', 'op': 'less_than', 'value': 0 }
                ]
            }
        }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_combination_1_false(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = {
            'NOT': {
                'AND': [
                    { 'key': 'id', 'op': 'greater_than', 'value': 0 },
                    { 'key': 'id', 'op': 'less_than', 'value': 2 }
                ]
            }
        }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_invalid_key(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = {
            'NOT': {
                'AND': [
                    { 'key': 1, 'op': 'greater_than', 'value': 0 },
                    { 'key': 'id', 'op': 'less_than', 'value': 2 }
                ]
            }
        }
        with self.assertRaises(ValueError) as context:
            meets_crit(record, criteria)
        
        self.assertEqual(str(context.exception), 'Invalid condition: {\'key\': 1, \'op\': \'greater_than\', \'value\': 0}')

    def test_invalid_value(self):
        record = { 'id': 1 , 'email': 'test@email.com', 'is_active': True }
        criteria = {
            'NOT': {
                'AND': [
                    { 'key': 'id', 'op': 'greater_than', 'value': None },
                    { 'key': 'id', 'op': 'less_than', 'value': 2 }
                ]
            }
        }
        with self.assertRaises(ValueError) as context:
            meets_crit(record, criteria)
        
        self.assertEqual(str(context.exception), 'Invalid condition: {\'key\': \'id\', \'op\': \'greater_than\', \'value\': None}')

    def test_not_equal_to_false(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'not_equal_to', 'value': 30 }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_not_equal_to_true(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'not_equal_to', 'value': 300 }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_greater_than_true(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'greater_than', 'value': 5 }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_greater_than_false(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'greater_than', 'value': 35 }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_greater_than_or_equal_to_true(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'greater_than_or_equal_to', 'value': 30 }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_greater_than_or_equal_to_false(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'greater_than_or_equal_to', 'value': 31 }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_less_than_or_equal_to_true(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'less_than_or_equal_to', 'value': 30 }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_less_than_or_equal_to_false(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'less_than_or_equal_to', 'value': 29 }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_less_than_true(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'less_than', 'value': 31 }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_less_than_false(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'less_than', 'value': 30 }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

    def test_in_true(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'in', 'value': [28, 29, 30, 31] }
        result = meets_crit(record, criteria)
        self.assertTrue(result)

    def test_in_false(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = {'key': 'age', 'op': 'in', 'value': [28, 29, 31] }
        result = meets_crit(record, criteria)
        self.assertFalse(result)

if __name__ == '__main__':
    unittest.main()



import unittest

from src.json_criteria import combine_criteria

class TestCombineCriteria(unittest.TestCase):
    def test_combine_and(self):
        criteria1 = { 'key': 'name', 'op': 'starts_with', 'value': 'S' }
        criteria2 = { 'key': 'age', 'op': 'equal_to', 'value': 100 }
        result = combine_criteria('AND', [criteria1, criteria2])
        self.assertDictEqual(result, {
            'AND': [
                { 'key': 'name', 'op': 'starts_with', 'value': 'S' },
                { 'key': 'age', 'op': 'equal_to', 'value': 100 }
            ]
        })

    def test_combine_or(self):
        criteria1 = { 'key': 'name', 'op': 'starts_with', 'value': 'S' }
        criteria2 = { 'key': 'age', 'op': 'equal_to', 'value': 100 }
        result = combine_criteria('OR', [criteria1, criteria2])
        self.assertDictEqual(result, {
            'OR': [
                { 'key': 'name', 'op': 'starts_with', 'value': 'S' },
                { 'key': 'age', 'op': 'equal_to', 'value': 100 }
            ]
        })

    def test_combine_and_2(self):
        criteria1 = {
            'AND': [
                { 'key': 'name', 'op': 'starts_with', 'value': 'S' },
                { 'key': 'age', 'op': 'equal_to', 'value': 100 }
            ]
        }
        criteria2 = {
            'OR': [
                { 'key': 'name', 'op': 'starts_with', 'value': 'L' },
                { 'key': 'age', 'op': 'equal_to', 'value': 20 }
            ]
        }
        result = combine_criteria('AND', [criteria1, criteria2])
        self.assertDictEqual(result, {
            'AND': [
                { 'AND': [{ 'key': 'name', 'op': 'starts_with', 'value': 'S' }, { 'key': 'age', 'op': 'equal_to', 'value': 100 }] },
                { 'OR': [{ 'key': 'name', 'op': 'starts_with', 'value': 'L' }, { 'key': 'age', 'op': 'equal_to', 'value': 20 }] }
            ]
        })

    def test_combine_or_2(self):
        criteria1 = {
            'AND': [
                { 'key': 'name', 'op': 'starts_with', 'value': 'S' },
                { 'key': 'age', 'op': 'equal_to', 'value': 100 }
            ]
        }
        criteria2 = {
            'OR': [
                { 'key': 'name', 'op': 'starts_with', 'value': 'L' },
                { 'key': 'age', 'op': 'equal_to', 'value': 20 }
            ]
        }
        result = combine_criteria('OR', [criteria1, criteria2])
        self.assertDictEqual(result, {
            'OR': [
                { 'AND': [{ 'key': 'name', 'op': 'starts_with', 'value': 'S' }, { 'key': 'age', 'op': 'equal_to', 'value': 100 }] },
                { 'OR': [{ 'key': 'name', 'op': 'starts_with', 'value': 'L' }, { 'key': 'age', 'op': 'equal_to', 'value': 20 }] }
            ]
        })

if __name__ == '__main__':
    unittest.main()



from typing import Dict, Any, List, Literal

def combine_criteria(
    operator: Literal['AND', 'OR'],
    criteria_list: List[Dict[str, Any]]
) -> Dict[str, Any]:
    """
    Combine multiple criteria using AND or OR operator.

    Parameters:
    - operator (Literal['AND', 'OR']): The operator to use when combining.
    - criteria_list (List[Dict[str, Any]]): The list of criteria to combine.

    Returns:
    - Dict[str, Any]: The new combined criteria.
    """
    if not criteria_list:
        # Return an empty criteria if the list is empty
        return {}

    # Validate that the operator is either 'AND' or 'OR'
    if operator not in ['AND', 'OR']:
        raise ValueError("Invalid operator. Use 'AND' or 'OR'.")

    # If there is only one criteria, return it directly
    if len(criteria_list) == 1:
        return criteria_list[0]

    # Combine multiple criteria using the specified operator
    return {operator: criteria_list}



from typing import Dict, Any
# from datetime import datetime
import re

def meets_crit(record: Dict[str, Any], element: Dict[str, Any]) -> bool:
    """
    Internal function to check if a record meets the given criteria.

    Parameters:
    - record (Dict[str, Any]): The data record.
    - element (Dict[str, Any]): The element of the criteria.

    Returns:
    - bool: Whether or not the record meets the criteria.
    """

    if not isinstance(element, dict):
        raise ValueError('Invalid element: must be a dictionary')

    # AND, OR, NOT operators
    if 'AND' in element:
        return all(meets_crit(record, el) for el in element.get('AND', []))
    if 'OR' in element:
        return any(meets_crit(record, el) for el in element.get('OR', []))
    if 'NOT' in element:
        return not meets_crit(record, element.get('NOT', {}))

    # Conditions
    key = element.get('key')
    operator = element.get('op')
    value = element.get('value')

    if not isinstance(key, str) or not operator or value is None:
        raise ValueError(f'Invalid condition: {element}')

    # Operator conditions - WIP
    # Change to dictionary where key is operator, value is compare function,
    # and break out into another module.
    if operator == 'equal_to':
        return record.get(key) == value
    if operator == 'not_equal_to':
        return record.get(key) != value
    if operator == 'greater_than':
        return record.get(key) > value
    if operator == 'less_than':
        return record.get(key) < value
    if operator == 'greater_than_or_equal_to':
        return record.get(key) >= value
    if operator == 'less_than_or_equal_to':
        return record.get(key) <= value
    if operator == 'in':
        return record.get(key) in value
    if operator == 'not_in':
        return record.get(key) not in value
    if operator == 'contains':
        return value in record.get(key)
    if operator == 'not_contains':
        return value not in record.get(key)
    if operator == 'starts_with':
        return record.get(key, '').startswith(value)
    if operator == 'not_starts_with':
        return not record.get(key, '').startswith(value)
    if operator == 'ends_with':
        return record.get(key, '').endswith(value)
    if operator == 'not_ends_with':
        return not record.get(key, '').endswith(value)
    # if operator == 'is_empty':
    #     return not bool(record.get(key))
    # if operator == 'is_not_empty':
    #     return bool(record.get(key))
    # if operator == 'is_true':
    #     return bool(record.get(key))
    # if operator == 'is_false':
    #     return not bool(record.get(key))
    # if operator == 'is_null':
    #     return record.get(key) is None
    # if operator == 'is_not_null':
    #     return record.get(key) is not None
    if operator == 'matches_regex':
        return re.search(value, str(record.get(key))) is not None
    # if operator == 'equals_ignore_case':
    #     return record.get(key, '').lower() == value.lower()
    # if operator == 'not_equals_ignore_case':
    #     return record.get(key, '').lower() != value.lower()
    # if operator == 'is_date':
    #     try:
    #         parsed_date = datetime.strptime(record.get(key, ''), '%Y-%m-%d')
    #         return True
    #     except ValueError:
    #         return False
    # if operator == 'date_equal_to':
    #     try:
    #         parsed_date = datetime.strptime(record.get(key, ''), '%Y-%m-%d')
    #         return parsed_date == value
    #     except ValueError:
    #         return False
    # if operator == 'date_before':
    #     try:
    #         parsed_date = datetime.strptime(record.get(key, ''), '%Y-%m-%d')
    #         return parsed_date < value
    #     except ValueError:
    #         return False
    # if operator == 'date_after':
    #     try:
    #         parsed_date = datetime.strptime(record.get(key, ''), '%Y-%m-%d')
    #         return parsed_date > value
    #     except ValueError:
    #         return False

    raise ValueError(f'Invalid operator: {operator}')
